@@ -15,7 +15,7 @@
 
 /*
    Modified by Timothy Mann, 1996
-   Last modified on Sat Apr 25 00:55:03 PDT 1998 by mann
+   Last modified on Sat Dec 12 11:47:51 PST 1998 by mann
 */
 
 #include "z80.h"
@@ -29,6 +29,7 @@
 
 #ifdef READLINE
 #include <readline/readline.h>
+#include <readline/history.h>
 #endif
 
 /*SUPPRESS 112*/
@@ -47,7 +48,74 @@ static Uchar *traps;
 static int num_traps;
 static int print_instructions;
 static int stop_signaled;
-extern char help_message[];  /* defined at the end */
+
+static char help_message[] =
+
+"(zbx) commands:\n\
+\n\
+Running:\n\
+    run\n\
+        Reset the Z-80 and commence execution at address 0000.\n\
+    cont\n\
+        Continue execution.\n\
+    step\n\
+        Execute one instruction, disallowing interrupts.\n\
+    stepint\n\
+        Execute one instruction, allowing an interrupt afterwards.\n\
+    next\n\
+    nextint\n\
+        Execute one instruction.  If the instruction is a CALL, continue\n\
+        until the return.  Interrupts are always allowed inside the call,\n\
+        but only the nextint form allows an interrupt afterwards.\n\
+    reset\n\
+        Reset the Z-80.\n\
+    timeroff\n\
+    timeron\n\
+        Disable/enable the emulated TRS-80 real time clock interrupt.\n\
+Printing:\n\
+    dump\n\
+        Print the values of the Z-80 registers.\n\
+    list\n\
+    list <addr>\n\
+    list <start addr> , <end addr>\n\
+        Disassemble 10 instructions at the current pc, 10 instructions at\n\
+        the specified hex address, or the instructions in the range of hex\n\
+        addresses.\n\
+    <start addr> , <end addr> /\n\
+    <start addr> / <num bytes>\n\
+    <addr> =\n\
+        Print the memory values in the specified range.  All values are hex.\n\
+    traceon\n\
+        Enable tracing of all instructions.\n\
+    traceoff\n\
+        Disable tracing.\n\
+    diskdump\n\
+        Print the state of the floppy disk controller emulation.\n\
+Traps:\n\
+    status\n\
+        Show all traps (breakpoints, trace points).\n\
+    clear\n\
+        Delete the trap at the current address.\n\
+    delete <n>\n\
+    delete *\n\
+        Delete trap n, or all traps.\n\
+    stop at <address>\n\
+        Set a breakpoint at the specified hex address.\n\
+    trace <address>\n\
+        Set a trap to trace execution at the specified hex address.\n\
+    traceon at <address>\n\
+        Set a trap to enable tracing at the specified hex address.\n\
+    traceoff at <address>\n\
+        Set a trap to disable tracing at the specified hex address.\n\
+Miscellaneous:\n\
+    assign $<reg> = <value>\n\
+    assign <addr> = <value>\n\
+        Change the value of a register, register pair, or memory byte.\n\
+    help\n\
+    ?\n\
+        Print this message.\n\
+    quit\n\
+        Exit the zbx.\n";
 
 static struct
 {
@@ -165,23 +233,30 @@ static void clear_trap_address(int address, int flag)
 
 void debug_print_registers()
 {
-    printf("\n       S Z   H  PV N C\n");
-    printf("Flags: %d %d   %d   %d %d %d\n\n",
+    printf("\n       S Z - H - PV N C   IFF1 IFF2 IM\n");
+    printf("Flags: %d %d %d %d %d  %d %d %d     %d    %d   %d\n\n",
 	   (SIGN_FLAG != 0),
 	   (ZERO_FLAG != 0),
+	   (REG_F & UNDOC5_MASK) != 0,
 	   (HALF_CARRY_FLAG != 0),
+	   (REG_F & UNDOC3_MASK) != 0,
 	   (OVERFLOW_FLAG != 0),
 	   (SUBTRACT_FLAG != 0),
-	   (CARRY_FLAG != 0));
+	   (CARRY_FLAG != 0),
+	   z80_state.iff1, z80_state.iff2, z80_state.interrupt_mode);
 
-    printf("A F: %.2x %.2x    IX: %.4x        AF': %.4x\n",
+    printf("A F: %.2x %.2x    IX: %.4x    AF': %.4x\n",
 	   REG_A, REG_F, REG_IX, REG_AF_PRIME);
-    printf("B C: %.2x %.2x    IY: %.4x        BC': %.4x\n",
+    printf("B C: %.2x %.2x    IY: %.4x    BC': %.4x\n",
 	   REG_B, REG_C, REG_IY, REG_BC_PRIME);
-    printf("D E: %.2x %.2x    PC: %.4x        DE': %.4x\n",
+    printf("D E: %.2x %.2x    PC: %.4x    DE': %.4x\n",
 	   REG_D, REG_E, REG_PC, REG_DE_PRIME);
-    printf("H L: %.2x %.2x    SP: %.4x        HL': %.4x\n",
+    printf("H L: %.2x %.2x    SP: %.4x    HL': %.4x\n",
 	   REG_H, REG_L, REG_SP, REG_HL_PRIME);
+
+    printf("\nT-state counter: %" TSTATE_T_LEN "    ", z80_state.t_count);
+    printf("Delay setting: %d (%s)\n",
+	   z80_state.delay, trs_autodelay ? "auto" : "fixed");
 }
 
 
@@ -302,6 +377,7 @@ void debug_shell()
     char input[MAXLINE];
     char command[MAXLINE];
     int done = 0;
+    sigset_t set, oldset;
 
 #ifdef READLINE
     char *line;
@@ -319,6 +395,11 @@ void debug_shell()
 	printf("\n");
 	disassemble(REG_PC);
 
+	sigemptyset(&set);
+	sigaddset(&set, SIGALRM);
+	sigaddset(&set, SIGIO);
+	sigprocmask(SIG_BLOCK, &set, &oldset);
+
 #ifdef READLINE
 	/*
 	 * Use the way cool gnu readline() utility.  Get completion,
@@ -346,16 +427,17 @@ void debug_shell()
 	    }
 	    else
 	    {
-		done = 1;
+		break;
 	    }
 	}
 #else
 	printf("(zbx) ");  fflush(stdout);
 
-	if (fgets(input, MAXLINE, stdin) == NULL) 
-	  done = 1;
+	if (fgets(input, MAXLINE, stdin) == NULL) break;
 #endif
 
+	sigprocmask(SIG_SETMASK, &oldset, NULL);
+
 	if(sscanf(input, "%s", command))
 	{
 	    if(!strcmp(command, "help") || !strcmp(command, "?"))
@@ -635,6 +717,10 @@ void debug_shell()
 	        /* Turn off emulated real time clock interrupt */
 	        trs_timer_on();
             }
+	    else if(!strcmp(command, "diskdump"))
+	    {
+		trs_disk_debug();
+	    }
 	    else
 	    {
 		int start_address, end_address, num_bytes;
@@ -663,72 +749,6 @@ void debug_shell()
 #endif
 }
 
-static char help_message[] =
-
-"(zbx) commands:\n\
-\n\
-Running:\n\
-    run\n\
-        Reset the Z-80 and commence execution at address 0000.\n\
-    cont\n\
-        Continue execution.\n\
-    step\n\
-        Execute one instruction, disallowing interrupts.\n\
-    stepint\n\
-        Execute one instruction, allowing an interrupt afterwards.\n\
-    next\n\
-    nextint\n\
-        Execute one instruction.  If the instruction is a CALL, continue\n\
-        until the return.  Interrupts are always allowed inside the call,\n\
-        but only the nextint form allows an interrupt afterwards.\n\
-    reset\n\
-        Reset the Z-80.\n\
-    timeroff\n\
-    timeron\n\
-        Disable/enable the emulated TRS-80 real time clock interrupt.\n\
-Printing:\n\
-    dump\n\
-        Print the values of the Z-80 registers.\n\
-    list\n\
-    list <addr>\n\
-    list <start addr> , <end addr>\n\
-        Disassemble 10 instructions at the current pc, 10 instructions at\n\
-        the specified hex address, or the instructions in the range of hex\n\
-        addresses.\n\
-    <start addr> , <end addr> /\n\
-    <start addr> / <num bytes>\n\
-    <addr> =\n\
-        Print the memory values in the specified range.  All values are hex.\n\
-    traceon\n\
-        Enable tracing of all instructions.\n\
-    traceoff\n\
-        Disable tracing.\n\
-Traps:\n\
-    status\n\
-        Show all traps (breakpoints, trace points).\n\
-    clear\n\
-        Delete the trap at the current address.\n\
-    delete <n>\n\
-    delete *\n\
-        Delete trap n, or all traps.\n\
-    stop at <address>\n\
-        Set a breakpoint at the specified hex address.\n\
-    trace <address>\n\
-        Set a trap to trace execution at the specified hex address.\n\
-    traceon at <address>\n\
-        Set a trap to enable tracing at the specified hex address.\n\
-    traceoff at <address>\n\
-        Set a trap to disable tracing at the specified hex address.\n\
-Miscellaneous:\n\
-    assign $<reg> = <value>\n\
-    assign <addr> = <value>\n\
-        Change the value of a register, register pair, or memory byte.\n\
-    help\n\
-    ?\n\
-        Print this message.\n\
-    quit\n\
-        Exit the zbx.\n";
-
 #ifdef NEVER
 
 /*
