@@ -18,14 +18,17 @@
    Last modified on Thu Sep 24 19:16:48 PDT 1998 by mann
 */
 
+#include <string.h>
+
 #include "z80.h"
 #include "trs.h"
 #include "trs_disk.h"
 #include "load_cmd.h"
 
 int trs_model = 1;
-int trs_pausing = 0;
+int trs_paused = 1;
 int trs_autodelay = 0;
+char *program_name;
 
 static void check_endian()
 {
@@ -34,7 +37,7 @@ static void check_endian()
     x.byte.high = 0;
     if(x.word != 1)
     {
-	fatal("Program compiled with wrong ENDIAN value -- adjust the Makefile.local, recompile, and try again.");
+	fatal("Program compiled with wrong ENDIAN value -- adjust the Makefile.local, type \"rm *.o\", recompile, and try again.");
     }
 }
 
@@ -42,7 +45,7 @@ void trs_load_rom(char *filename)
 {
     FILE *program;
     int c;
-    
+
     if((program = fopen(filename, "r")) == NULL)
     {
 	char message[100];
@@ -103,9 +106,16 @@ int main(int argc, char *argv[])
 
     check_endian();
 
+    program_name = strrchr(argv[0], '/');
+    if (program_name == NULL) {
+      program_name = argv[0];
+    } else {
+      program_name++;
+    }
+
     argc = trs_parse_command_line(argc, argv, &debug);
     if (argc > 1) {
-      fprintf(stderr, "%s: erroneous argument %s\n", argv[0], argv[1]);
+      fprintf(stderr, "%s: erroneous argument %s\n", program_name, argv[1]);
       exit(1);
     }
     mem_init();
