@@ -27,10 +27,13 @@
 #define NORMAL 0
 #define EXPANDED 1
 #define INVERSE 2
+#define ALTERNATE 4
 
 extern int trs_model; /* 1, 3, 4, 5(=4p) */
-extern int trs_pausing;
+extern int trs_paused;
 extern int trs_autodelay;
+void trs_suspend_delay(void);
+void trs_restore_delay(void);
 
 extern int trs_parse_command_line(int argc, char **argv, int *debug);
 
@@ -38,6 +41,7 @@ extern void trs_screen_init(void);
 extern void trs_screen_write_char(int position, int char_index, int doflush);
 extern void trs_screen_write_chars(int *locations, int *values, int count);
 extern void trs_screen_expanded(int flag);
+extern void trs_screen_alternate(int flag);
 extern void trs_screen_80x24(int flag);
 extern void trs_screen_inverse(int flag);
 extern void trs_screen_scroll(void);
@@ -63,7 +67,7 @@ extern int trs_printer_read(void);
 
 extern void trs_cassette_motor(int value);
 extern void trs_cassette_out(int value);
-extern int trs_cassette_in(int modesel);
+extern int trs_cassette_in(void);
 extern void trs_sound_out(int value);
 extern void trs_sound_init(int ioport, int vol);
 
@@ -90,9 +94,17 @@ extern void trs_timer_init(void);
 extern void trs_timer_off(void);
 extern void trs_timer_on(void);
 extern void trs_timer_speed(int flag);
+extern void trs_cassette_rise_interrupt(int dummy);
+extern void trs_cassette_fall_interrupt(int dummy);
+extern void trs_cassette_clear_interrupts(void);
+extern int trs_cassette_interrupts_enabled(void);
+extern void trs_cassette_update(int dummy);
+extern int cassette_default_sample_rate;
 
 extern void trs_disk_change(int drive);
-extern void trs_disk_change_all();
+extern void trs_disk_change_all(void);
+extern void trs_disk_debug(void);
+extern int trs_disk_motoroff(void);
 
 extern void mem_video_page(int which);
 extern void mem_bank(int which);
@@ -102,9 +114,11 @@ extern void mem_romin(int state);
 extern void trs_debug(void);
 
 typedef void (*trs_event_func)(int arg);
-void trs_schedule_event(trs_event_func f, int arg, int when);
+void trs_schedule_event(trs_event_func f, int arg, int tstates);
+void trs_schedule_event_us(trs_event_func f, int arg, int us);
 void trs_do_event(void);
 void trs_cancel_event(void);
+trs_event_func trs_event_scheduled(void);
 
 void grafyx_write_x(int value);
 void grafyx_write_y(int value);
@@ -115,9 +129,13 @@ void grafyx_write_xoffset(int value);
 void grafyx_write_yoffset(int value);
 void grafyx_write_overlay(int value);
 void grafyx_set_microlabs(int on_off);
+int grafyx_get_microlabs(void);
 
 void trs_get_mouse_pos(int *x, int *y, unsigned int *buttons);
 void trs_set_mouse_pos(int x, int y);
 void trs_get_mouse_max(int *x, int *y, unsigned int *sens);
 void trs_set_mouse_max(int x, int y, unsigned int sens);
-int trs_get_mouse_type();
+int trs_get_mouse_type(void);
+
+void sb_set_volume(int vol);
+int sb_get_volume(void);
