@@ -5,7 +5,7 @@
  * retained, and (2) modified versions are clearly marked as having
  * been modified, with the modifier's name and the date included.  */
 
-/* Last modified on Mon Sep 28 12:27:21 PDT 1998 by mann */
+/* Last modified on Tue Dec 15 11:34:26 PST 1998 by mann */
 
 /*
  * trs_imp_exp.c
@@ -105,6 +105,28 @@ void do_emt_mouse()
   }
 }
 
+void do_emt_getddir()
+{
+  if (REG_HL + REG_BC > 0x10000 ||
+      REG_HL + strlen(trs_disk_dir) + 1 > REG_HL + REG_BC) {
+    REG_A = EFAULT;
+    REG_F &= ~ZERO_MASK;
+    REG_BC = 0xFFFF;
+    return;
+  }
+  strcpy(mem_pointer(REG_HL, 1), trs_disk_dir);
+  REG_A = 0;
+  REG_F |= ZERO_MASK;
+  REG_BC = strlen(trs_disk_dir);
+}
+
+void do_emt_setddir()
+{
+  trs_disk_dir = strdup(mem_pointer(REG_HL, 0));
+  REG_A = 0;
+  REG_F |= ZERO_MASK;
+}
+
 void do_emt_open()
 {
   int fd, oflag, eoflag;
@@ -427,6 +449,63 @@ void do_emt_misc()
   case 4:
     REG_HL = trs_disk_changecount;
     break;
+  case 5:
+    REG_HL = trs_model;
+    break;
+  case 6:
+    REG_HL = trs_disk_getsize(REG_BC);
+    break;
+  case 7:
+    trs_disk_setsize(REG_BC, REG_HL);
+    break;
+  case 8:
+    REG_HL = trs_disk_getstep(REG_BC);
+    break;
+  case 9:
+    trs_disk_setstep(REG_BC, REG_HL);
+    break;
+  case 10:
+    REG_HL = grafyx_get_microlabs();
+    break;
+  case 11:
+    grafyx_set_microlabs(REG_HL);
+    break;
+  case 12:
+    REG_HL = z80_state.delay;
+    REG_BC = trs_autodelay;
+    break;
+  case 13:
+    z80_state.delay = REG_HL;
+    trs_autodelay = REG_BC;
+    break;
+  case 14:
+    REG_HL = stretch_amount;
+    REG_BC = stretch_poll;
+    REG_DE = stretch_heartbeat;
+    break;
+  case 15:
+    stretch_amount = REG_HL;
+    stretch_poll = REG_BC;
+    stretch_heartbeat = REG_DE;
+    break;
+  case 16:
+    REG_HL = trs_disk_doubler;
+    break;
+  case 17:
+    trs_disk_doubler = REG_HL;
+    break;
+  case 18:
+    REG_HL = sb_get_volume();
+    break;
+  case 19:
+    sb_set_volume(REG_HL);
+    break;
+  case 20:
+    REG_HL = trs_disk_truedam;
+    break;
+  case 21:
+    trs_disk_truedam = REG_HL;
+    break;
   default:
     error("unsupported function code to emt_misc");
     break;
@@ -484,7 +563,7 @@ void do_emt_opendisk()
   if (*name == '/' || *trs_disk_dir == '\0') {
     qname = strdup(name);
   } else {
-    qname = (char *)malloc(strlen(trs_disk_dir) + 1 + strlen(name));
+    qname = (char *)malloc(strlen(trs_disk_dir) + 1 + strlen(name) + 1);
     strcpy(qname, trs_disk_dir);
     strcat(qname, "/");
     strcat(qname, name);
