@@ -5,7 +5,7 @@
  * retained, and (2) modified versions are clearly marked as having
  * been modified, with the modifier's name and the date included.  */
 
-/* Last modified on Thu Sep 24 19:16:50 PDT 1998 by mann */
+/* Last modified on Sat Dec 12 11:46:26 PST 1998 by mann */
 
 /*
  * Emulate interrupts
@@ -15,12 +15,15 @@
 #include "trs.h"
 #include <stdio.h>
 #include <sys/time.h>
+#include <time.h>
 #include <signal.h>
 
 /* IRQs */
-#define M1_TIMER_BIT 0x80
-#define M1_DISK_BIT  0x40
-#define M3_TIMER_BIT 0x04
+#define M1_TIMER_BIT    0x80
+#define M1_DISK_BIT     0x40
+#define M3_TIMER_BIT    0x04
+#define M3_CASSFALL_BIT 0x02
+#define M3_CASSRISE_BIT 0x01
 static unsigned char interrupt_latch = 0;
 static unsigned char interrupt_mask = 0;
 
@@ -57,6 +60,43 @@ static int timer_on = 1;
 long lost_timer_interrupts = 0;
 #endif
 
+/* Note: the independent interrupt latch and mask model is not correct
+   for all interrupts.  The cassette rise/fall interrupt enable is
+   clocked into the interrupt latch when the event occurs (we get this
+   right), and is *not* masked against the latch output (we get this
+   wrong, but it doesn't really matter). */
+
+void
+trs_cassette_rise_interrupt(int dummy)
+{
+  interrupt_latch = (interrupt_latch & ~M3_CASSRISE_BIT) |
+    (interrupt_mask & M3_CASSRISE_BIT);
+  z80_state.irq = (interrupt_latch & interrupt_mask) != 0;
+  trs_cassette_update(0);
+}
+
+void
+trs_cassette_fall_interrupt(int dummy)
+{
+  interrupt_latch = (interrupt_latch & ~M3_CASSFALL_BIT) |
+    (interrupt_mask & M3_CASSFALL_BIT);
+  z80_state.irq = (interrupt_latch & interrupt_mask) != 0;
+  trs_cassette_update(0);
+}
+
+void
+trs_cassette_clear_interrupts()
+{
+  interrupt_latch &= ~(M3_CASSRISE_BIT|M3_CASSFALL_BIT);
+  z80_state.irq = (interrupt_latch & interrupt_mask) != 0;
+}
+
+int
+trs_cassette_interrupts_enabled()
+{
+  return interrupt_mask & (M3_CASSRISE_BIT|M3_CASSFALL_BIT);
+}
+
 void
 trs_timer_interrupt(int state)
 {
@@ -66,10 +106,10 @@ trs_timer_interrupt(int state)
       if (interrupt_latch & M1_TIMER_BIT) lost_timer_interrupts++;
 #endif
       interrupt_latch |= M1_TIMER_BIT;
+      z80_state.irq = 1;
     } else {
       interrupt_latch &= ~M1_TIMER_BIT;
     }
-    z80_state.irq = (interrupt_latch != 0);
   } else {
     if (state) {
 #ifdef IDEBUG
@@ -89,10 +129,10 @@ trs_disk_intrq_interrupt(int state)
   if (trs_model == 1) {
     if (state) {
       interrupt_latch |= M1_DISK_BIT;
+      z80_state.irq = 1;
     } else {
       interrupt_latch &= ~M1_DISK_BIT;
     }
-    z80_state.irq = (interrupt_latch != 0);
   } else {
     if (state) {
       nmi_latch |= M3_INTRQ_BIT;
@@ -107,11 +147,9 @@ trs_disk_intrq_interrupt(int state)
 void
 trs_disk_motoroff_interrupt(int state)
 {
-  /* Drive motor timed out (stopped).
-     Not emulated; this routine is never called.
-   */
+  /* Drive motor timed out (stopped). */
   if (trs_model == 1) {
-    /* no effect */
+    /* no such interrupt */
   } else {
     if (state) {
       nmi_latch |= M3_MOTOROFF_BIT;
@@ -151,6 +189,7 @@ trs_interrupt_latch_read()
   unsigned char tmp = interrupt_latch;
   if (trs_model == 1) {
     trs_timer_interrupt(0); /* acknowledge this one (only) */
+    z80_state.irq = (interrupt_latch != 0);
     return tmp;
   } else {
     return ~tmp;
@@ -179,6 +218,33 @@ trs_nmi_mask_write(unsigned char value)
   if (!z80_state.nmi) z80_state.nmi_seen = 0;
 }
 
+static int saved_delay;
+
+/* Temporarily reduce the delay, until trs_restore_delay is called.
+   Useful if we know we're about to do something that's emulated more
+   slowly than most instructions, such as video or real-time sound.
+   In case the boost is too big or too small, we allow the normal
+   autodelay algorithm to continue to run and adjust the new delay. */
+void
+trs_suspend_delay()
+{
+  if (!saved_delay) {
+    saved_delay = z80_state.delay;
+    z80_state.delay /= 2;  /* dividing by 2 is arbitrary */
+  }
+}
+
+/* Put back the saved delay */
+void
+trs_restore_delay()
+{
+  if (saved_delay) {
+    z80_state.delay = saved_delay;
+    saved_delay = 0;
+    trs_paused = 1;
+  }
+}
+
 #define UP_F   1.50
 #define DOWN_F 0.50 
 
@@ -200,7 +266,7 @@ trs_timer_event(int signo)
 #else
       static unsigned long oldtcount;
 #endif
-      if (!trs_pausing) {
+      if (!trs_paused /*&& !saved_delay*/) {
 	int toofast = (z80_state.t_count - oldtcount) >
 	  ((tv.tv_sec*1000000 + tv.tv_usec) -
 	   (oldtv.tv_sec*1000000 + oldtv.tv_usec))*z80_state.clockMHz;
@@ -221,12 +287,14 @@ trs_timer_event(int signo)
 	  }
 	}
       }
+      trs_paused = 0;
       oldtv = tv;
       oldtcount = z80_state.t_count;
   }
 
   trs_timer_interrupt(1); /* generate */
   trs_kb_heartbeat(); /* part of keyboard stretch kludge */
+  trs_disk_motoroff_interrupt(trs_disk_motoroff());
 #if HAVE_SIGIO
   x_poll_count = 0; /* be sure to flush X events */
 #endif
@@ -316,7 +384,7 @@ trs_timer_speed(int fast)
 static trs_event_func event_func = NULL;
 static int event_arg;
 
-/* Schedule an event to occur after "countdown" more instructions have
+/* Schedule an event to occur after "countdown" more t-states have
  *  executed.  0 makes the event happen immediately -- that is, at
  *  the end of the current instruction, but before the emulator checks
  *  for interrupts.  It is legal for an event function to call 
@@ -337,7 +405,8 @@ trs_schedule_event(trs_event_func f, int arg, int countdown)
     }
     event_func = f;
     event_arg = arg;
-    z80_state.sched = countdown;
+    z80_state.sched = z80_state.t_count + (tstate_t) countdown;
+    if (z80_state.sched == 0) z80_state.sched--;
 }
 
 /*
@@ -350,7 +419,7 @@ trs_do_event()
     trs_event_func f = event_func;
     if (f) {
 	event_func = NULL;
-	z80_state.sched = -1;
+	z80_state.sched = 0;
 	f(event_arg);    
     }
 }
@@ -362,5 +431,14 @@ void
 trs_cancel_event()
 {
     event_func = NULL;
-    z80_state.sched = -1;
+    z80_state.sched = 0;
+}
+
+/*
+ * Check event scheduled
+ */
+trs_event_func
+trs_event_scheduled()
+{
+    return event_func;
 }
