@@ -24,7 +24,7 @@
 #include "trs_disk.h"
 
 static int modesel = 0;     /* Model I */
-static int modeimage = 0;   /* Model III/4/4p */
+static int modeimage = 0x8; /* Model III/4/4p */
 static int ctrlimage = 0;   /* Model 4/4p */
 static int rominimage = 0;  /* Model 4p */
 
@@ -39,6 +39,10 @@ void z80_out(int port, int value)
 	  case IMPEXP_DATA: /* 0xD1 */
 	    trs_impexp_data_write(value);
 	    break;
+	  case 0xFD:
+	    /* GENIE location of printer port */
+	    trs_printer_write(value);
+	    break;
 	  case 0xFF:
 	    /* screen mode select is on D3 line */
 	    modesel = (value >> 3) & 1;
@@ -136,9 +140,10 @@ void z80_out(int port, int value)
             trs_cassette_motor((modeimage & 0x02) >> 1);
 	    /* screen mode select is on D2 */
 	    trs_screen_expanded((modeimage & 0x04) >> 2);
+	    /* alternate char set is on D3 */
+	    trs_screen_alternate(!((modeimage & 0x08) >> 3));
 	    /* clock speed is on D6; it affects timer HZ too */
 	    trs_timer_speed((modeimage & 0x40) >> 6);
-	    /* !! still to do: alt char set */
 	    break;
 	  case TRSDISK3_COMMAND: /* 0xF0 */
 	    trs_disk_command_write(value);
@@ -187,8 +192,11 @@ int z80_in(int port)
 	    return trs_impexp_status_read();
 	  case IMPEXP_DATA: /* 0xD1 */
 	    return trs_impexp_data_read();
+	  case 0xFD:
+	    /* GENIE location of printer port */
+	    return trs_printer_read();
 	  case 0xFF:
-	    return trs_cassette_in(modesel);
+	    return (modesel ? 0x7f : 0x3f) | trs_cassette_in();
 	  default:
 	    break;
 	}
@@ -232,8 +240,7 @@ int z80_in(int port)
 	  case 0xF8:
 	    return trs_printer_read();
 	  case 0xFF:
-	    /* !!Add cassette bits, etc., later */
-	    return modeimage;
+	    return (modeimage & 0x7e) | trs_cassette_in();
 	  default:
 	    break;
 	}
