@@ -15,7 +15,7 @@
 
 /*
    Modified by Timothy Mann, 1996
-   Last modified on Fri Sep 25 19:49:40 PDT 1998 by mann
+   Last modified on Mon Jul 26 12:52:24 PDT 1999 by mann
 */
 
 /*
@@ -148,7 +148,7 @@ void mem_bank(int command)
 	bank_offset[1] = 0 << 15;
 	break;
       default:
-	fprintf(stderr, "unknown mem_bank command %d\n", command);
+	error("unknown mem_bank command %d", command);
 	break;
     }
 }
@@ -176,13 +176,13 @@ void trs_reset()
     }
     trs_kb_reset();  /* Part of keyboard stretch kludge */
 
-    if (trs_model == 5) {
-	/* Reset processor (!!guess this is what 4P does) */
+    if (trs_model >= 4) {
+        /* Reset processor */
 	z80_reset();
     } else {
 	/* Signal a nonmaskable interrupt. */
 	trs_reset_button_interrupt(1);
-	trs_schedule_event(trs_reset_button_interrupt, 0, 500);
+	trs_schedule_event(trs_reset_button_interrupt, 0, 2000);
     }
 }
 
@@ -320,7 +320,7 @@ void mem_write(int address, int value)
 	    memory[address] = value;
 	} else if (address >= VIDEO_START) {
 	    int vaddr = address + video_offset;
-#ifdef UPPERCASE
+#if UPPERCASE
 	    /*
 	     * Video write.  Hack here to make up for the missing bit 6
 	     * video ram, emulating the gate in Z30.
@@ -523,16 +523,17 @@ Uchar *mem_pointer(int address, int writing)
 /*
  * Block move instructions, for LDIR and LDDR instructions.
  *
- * Direction is either +1 or -1.
+ * Direction is either +1 or -1.  
  *
  * Note that a count of zero => move 64K bytes.
  *
  * These can be special cased to do fun stuff like fast
  * video scrolling.
  */
-void
+int
 mem_block_transfer(Ushort dest, Ushort source, int direction, Ushort count)
 {
+    int ret;
     /* special case for screen scroll */
     if(trs_model <= 3 &&
        (dest == VIDEO_START) && (source == VIDEO_START + 0x40) &&
@@ -542,7 +543,7 @@ mem_block_transfer(Ushort dest, Ushort source, int direction, Ushort count)
 	video_scroll();
 
 	do
-	  memory[dest++] = memory[source++];
+	  memory[dest++] = ret = memory[source++];
 	while(count--);
     }
     else
@@ -553,7 +554,7 @@ mem_block_transfer(Ushort dest, Ushort source, int direction, Ushort count)
 	{
 	    do
 	    {
-		mem_write(dest++, mem_read(source++));
+		mem_write(dest++, ret = mem_read(source++));
 		count--;
 	    }
 	    while(count);
@@ -562,7 +563,7 @@ mem_block_transfer(Ushort dest, Ushort source, int direction, Ushort count)
 	{
 	    do
 	    {
-		mem_write(dest--, mem_read(source--));
+		mem_write(dest--, ret = mem_read(source--));
 		count--;
 	    }
 	    while(count);
@@ -570,4 +571,6 @@ mem_block_transfer(Ushort dest, Ushort source, int direction, Ushort count)
 
 	uncache_video_writes();
     }
+    return ret;
 }
+
