@@ -15,7 +15,7 @@
 
 /*
    Modified by Timothy Mann, 1996
-   Last modified on Mon Oct 12 13:42:36 PDT 1998 by mann
+   Last modified on Mon Jul 26 12:52:10 PDT 1999 by mann
 */
 
 /*#define MOUSEDEBUG 1*/
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <malloc.h>
@@ -48,18 +49,14 @@
 #include "z80.h"
 #include "trs_disk.h"
 
-#define DEF_FONT1 \
-    "-trs80-model1-medium-r-normal--24-*-64-64-c-80-trs80-model1"
-#define DEF_WIDEFONT1 \
-    "-trs80-model1-medium-r-normal--24-*-64-64-c-160-trs80-model1"
-#define DEF_FONT3 \
-    "-trs80-model3-medium-r-normal--24-*-64-64-c-80-trs80-model3"
-#define DEF_WIDEFONT3 \
-    "-trs80-model3-medium-r-normal--24-*-64-64-c-160-trs80-model3"
+#define DEF_FONT1	"-misc-fixed-medium-r-normal--20-200-75-75-*-100-iso8859-1"
+#define DEF_WIDEFONT1	"-misc-fixed-medium-r-normal--20-200-75-75-*-200-iso8859-1"
+#define DEF_FONT3	"-misc-fixed-medium-r-normal--20-200-75-75-*-100-iso8859-1"
+#define DEF_WIDEFONT3	"-misc-fixed-medium-r-normal--20-200-75-75-*-200-iso8859-1"
 #define DEF_USEFONT 0
 
-extern char trs_char_data[2][MAXCHARS][TRS_CHAR_HEIGHT];
-extern char trs_widechar_data[2][MAXCHARS][TRS_CHAR_HEIGHT][2];
+extern char trs_char_data[][MAXCHARS][TRS_CHAR_HEIGHT];
+extern char trs_widechar_data[][MAXCHARS][TRS_CHAR_HEIGHT][2];
 
 #define EVENT_MASK \
   ExposureMask | KeyPressMask | MapRequest | KeyReleaseMask | \
@@ -75,6 +72,7 @@ static int top_margin = 0;
 static int left_margin = 0;
 static int border_width = 2;
 static Pixmap trs_char[2][MAXCHARS];
+static Pixmap trs_box[2][64];
 static Display *display;
 static int screen;
 static Window window;
@@ -84,12 +82,12 @@ static GC gc_xor;
 static int currentmode = NORMAL;
 static int OrigHeight,OrigWidth;
 static int usefont = DEF_USEFONT;
-static int trsfont;
 static int cur_char_width = TRS_CHAR_WIDTH;
 static int cur_char_height = TRS_CHAR_HEIGHT;
 static int text80x24 = 0, screen640x240 = 0;
 static XFontStruct *myfont, *mywidefont, *curfont;
 static XKeyboardState repeat_state;
+static int trs_charset;
 
 static XrmOptionDescRec opts[] = {
 {"-background",	"*background",	XrmoptionSepArg,	(caddr_t)NULL},
@@ -101,8 +99,6 @@ static XrmOptionDescRec opts[] = {
 {"-nofont",	"*usefont",	XrmoptionNoArg,		(caddr_t)"off"},
 {"-font",	"*font",	XrmoptionSepArg,	(caddr_t)NULL},
 {"-widefont",	"*widefont",	XrmoptionSepArg,	(caddr_t)NULL},
-{"-trsfont",	"*trsfont",	XrmoptionNoArg,		(caddr_t)"on"},
-{"-notrsfont",	"*trsfont",	XrmoptionNoArg,		(caddr_t)"off"},
 {"-display",	"*display",	XrmoptionSepArg,	(caddr_t)NULL},
 {"-debug",	"*debug",	XrmoptionNoArg,		(caddr_t)"on"},
 {"-nodebug",	"*debug",	XrmoptionNoArg,		(caddr_t)"off"},
@@ -126,6 +122,13 @@ static XrmOptionDescRec opts[] = {
 {"-microlabs",  "*microlabs",   XrmoptionNoArg,         (caddr_t)"on"},
 {"-nomicrolabs","*microlabs",   XrmoptionNoArg,         (caddr_t)"off"},
 {"-doubler",    "*doubler",     XrmoptionSepArg,        (caddr_t)NULL},
+{"-sizemap",    "*sizemap",     XrmoptionSepArg,        (caddr_t)NULL},
+{"-stepmap",    "*stepmap",     XrmoptionSepArg,        (caddr_t)NULL},
+{"-charset",    "*charset",     XrmoptionSepArg,        (caddr_t)NULL},
+{"-truedam",    "*truedam",     XrmoptionNoArg,         (caddr_t)"on"},
+{"-notruedam",  "*truedam",     XrmoptionNoArg,         (caddr_t)"off"},
+{"-samplerate", "*samplerate",  XrmoptionSepArg,        (caddr_t)11025},
+{"-title",      "*title",       XrmoptionSepArg,        (caddr_t)NULL},
 #if __linux
 {"-sb",         "*sb",          XrmoptionSepArg,        (caddr_t)NULL},
 #endif /* linux */
@@ -184,6 +187,9 @@ static int key_queue[KEY_QUEUE_SIZE];
 static int key_queue_head;
 static int key_queue_entries;
 
+/* dummy buffer for stat() call */
+struct stat statbuf;
+
 static void clear_key_queue()
 {
   key_queue_head = 0;
@@ -193,12 +199,12 @@ static void clear_key_queue()
 void queue_key(int state)
 {
   key_queue[(key_queue_head + key_queue_entries) % KEY_QUEUE_SIZE] = state;
-#ifdef KBDEBUG
+#if KBDEBUG
   fprintf(stderr, "queue_key 0x%x", state);
 #endif
   if (key_queue_entries < KEY_QUEUE_SIZE) {
     key_queue_entries++;
-#ifdef KBDEBUG
+#if KBDEBUG
     fprintf(stderr, "\n");
   } else {
     fprintf(stderr, " (overflow)\n");
@@ -215,7 +221,7 @@ int dequeue_key()
       rval = key_queue[key_queue_head];
       key_queue_head = (key_queue_head + 1) % KEY_QUEUE_SIZE;
       key_queue_entries--;
-#ifdef KBDEBUG
+#if KBDEBUG
       fprintf(stderr, "dequeue_key 0x%x\n", rval);
 #endif
     }
@@ -230,13 +236,13 @@ int trs_next_key(int wait)
     for (;;) {
       if ((rval = dequeue_key()) >= 0) break;
       if ((z80_state.nmi && !z80_state.nmi_seen) ||
-	  (z80_state.irq && z80_state.iff1)) {
+	  (z80_state.irq && z80_state.iff1) ||
+	  trs_event_scheduled()) {
 	rval = -1;
 	break;
       }
-      trs_pausing = 1;
+      trs_paused = 1;
       pause();			/* Wait for SIGALRM or SIGIO */
-      trs_pausing = 0;
       trs_get_event(0);
     }
     return rval;
@@ -254,7 +260,8 @@ void trs_event();
 
 static XrmDatabase x_db = NULL;
 static XrmDatabase command_db = NULL;
-static char *program_name;
+extern char *program_name;
+char *title;
 
 int trs_parse_command_line(int argc, char **argv, int *debug)
 {
@@ -262,14 +269,10 @@ int trs_parse_command_line(int argc, char **argv, int *debug)
   char *type;
   XrmValue value;
   char *xrms;
+  int stepdefault, i, s[8];
+
+  title = program_name; /* default */
 
-  program_name = strrchr(argv[0], '/');
-  if (program_name == NULL) {
-    program_name = argv[0];
-  } else {
-    program_name++;
-  }
-    
   XrmInitialize();
   /* parse command line options */
   XrmParseCommand(&command_db,opts,num_opts,program_name,&argc,argv);
@@ -323,15 +326,6 @@ int trs_parse_command_line(int argc, char **argv, int *debug)
     }
   }
 
-  (void) sprintf(option, "%s%s", program_name, ".doublestep");
-  if (XrmGetResource(x_db, option, "Xtrs.doublestep", &type, &value)) {
-    if (strcmp(value.addr,"on") == 0) {
-      trs_disk_doublestep = True;
-    } else if (strcmp(value.addr,"off") == 0) {
-      trs_disk_doublestep = False;
-    }
-  }
-
   (void) sprintf(option, "%s%s", program_name, ".model");
   if (XrmGetResource(x_db, option, "Xtrs.Model", &type, &value)) {
     if (strcmp(value.addr, "1") == 0 ||
@@ -347,9 +341,63 @@ int trs_parse_command_line(int argc, char **argv, int *debug)
 	       strcasecmp(value.addr, "IVp") == 0) {
       trs_model = 5;
     } else {
-      fprintf(stderr, "%s: TRS-80 Model %s not supported\n",
-	      program_name, value.addr);
-      exit(1);
+      fatal("TRS-80 Model %s not supported", value.addr);
+    }
+  }
+
+  /* !!Note: charset numbers must match trs_chars.c */
+  if (trs_model == 1) {
+    char *charset_name = "wider"; /* default */
+    (void) sprintf(option, "%s%s", program_name, ".charset");
+    if (XrmGetResource(x_db, option, "Xtrs.Charset", &type, &value)) {
+      charset_name = (char*) value.addr;
+    }
+    if (isdigit(*charset_name)) {
+      trs_charset = atoi(charset_name);
+    } else {
+      if (charset_name[0] == 'e'/*early*/) {
+	trs_charset = 0;
+	cur_char_width = 6;
+      } else if (charset_name[0] == 's'/*stock*/) {
+	trs_charset = 1;
+	cur_char_width = 6;
+      } else if (charset_name[0] == 'l'/*lcmod*/) {
+	trs_charset = 2;
+	cur_char_width = 6;
+      } else if (charset_name[0] == 'w'/*wider*/) {
+	trs_charset = 3;
+	cur_char_width = 8;
+      } else if (charset_name[0] == 'g'/*genie or german*/) {
+	trs_charset = 10;
+	cur_char_width = 8;
+      } else {
+	fatal("unknown charset name %s", value.addr);
+      }
+    }
+  } else /* trs_model > 1 */ {
+    char *charset_name;
+    /* Set default */
+    if (trs_model == 3) {
+      charset_name = "katakana";
+    } else {
+      charset_name = "international";
+    }
+    (void) sprintf(option, "%s%s", program_name, ".charset");
+    if (XrmGetResource(x_db, option, "Xtrs.Charset", &type, &value)) {
+      charset_name = (char*) value.addr;
+    }
+    if (isdigit(*charset_name)) {
+      trs_charset = atoi(charset_name);
+    } else {
+      if (charset_name[0] == 'k'/*katakana*/) {
+	trs_charset = 4 + 3*(trs_model > 3);
+      } else if (charset_name[0] == 'i'/*international*/) {
+	trs_charset = 5 + 3*(trs_model > 3);
+      } else if (charset_name[0] == 'b'/*bold*/) {
+	trs_charset = 6 + 3*(trs_model > 3);
+      } else {
+	fatal("unknown charset name %s", value.addr);
+      }
     }
   }
 
@@ -403,6 +451,74 @@ int trs_parse_command_line(int argc, char **argv, int *debug)
     }
   }
 
+  /* Defaults for sizemap */
+  s[0] = 5;
+  s[1] = 5;
+  s[2] = 5;
+  s[3] = 5;
+  s[4] = 8;
+  s[5] = 8;
+  s[6] = 8;
+  s[7] = 8;
+  (void) sprintf(option, "%s%s", program_name, ".sizemap");
+  if (XrmGetResource(x_db, option, "Xtrs.Sizemap", &type, &value)) {
+    sscanf((char*)value.addr, "%d,%d,%d,%d,%d,%d,%d,%d",
+	   &s[0], &s[1], &s[2], &s[3], &s[4], &s[5], &s[6], &s[7]);
+  }
+  for (i=0; i<=7; i++) {
+    if (s[i] != 5 && s[i] != 8) {
+      fatal("bad value %d for disk %d size", s[i], i);
+    } else {
+      trs_disk_setsize(i, s[i]);
+    }
+  }
+
+  /* Defaults for stepmap */
+  (void) sprintf(option, "%s%s", program_name, ".doublestep");
+  stepdefault = 1;
+  if (XrmGetResource(x_db, option, "Xtrs.doublestep", &type, &value)) {
+    if (strcmp(value.addr,"on") == 0) {
+      stepdefault = 2;
+    } else if (strcmp(value.addr,"off") == 0) {
+      stepdefault = 1;
+    }
+  }
+
+  for (i=0; i<=7; i++) {
+    s[i] = stepdefault;
+  }
+  (void) sprintf(option, "%s%s", program_name, ".stepmap");
+  if (XrmGetResource(x_db, option, "Xtrs.Stepmap", &type, &value)) {
+    sscanf((char*)value.addr, "%d,%d,%d,%d,%d,%d,%d,%d",
+	   &s[0], &s[1], &s[2], &s[3], &s[4], &s[5], &s[6], &s[7]);
+  }
+  for (i=0; i<=7; i++) {
+    if (s[i] != 1 && s[i] != 2) {
+      fatal("bad value %d for disk %d single/double step\n", s[i], i);
+    } else {
+      trs_disk_setstep(i, s[i]);
+    }
+  }
+
+  (void) sprintf(option, "%s%s", program_name, ".truedam");
+  if (XrmGetResource(x_db, option, "Xtrs.Truedam", &type, &value)) {
+    if (strcmp(value.addr,"on") == 0) {
+      trs_disk_truedam = True;
+    } else if (strcmp(value.addr,"off") == 0) {
+      trs_disk_truedam = False;
+    }
+  }
+
+  (void) sprintf(option, "%s%s", program_name, ".samplerate");
+  if (XrmGetResource(x_db, option, "Xtrs.Samplerate", &type, &value)) {
+    cassette_default_sample_rate = strtol(value.addr, NULL, 0);
+  }
+
+  (void) sprintf(option, "%s%s", program_name, ".title");
+  if (XrmGetResource(x_db, option, "Xtrs.title", &type, &value)) {
+      title = strdup(value.addr);
+  }
+
   return argc;
 }
 
@@ -411,6 +527,7 @@ save_repeat()
 {
   XGetKeyboardControl(display, &repeat_state);
   XAutoRepeatOff(display);
+  XSync(display, FALSE);
 }
 
 static void
@@ -437,6 +554,7 @@ void trs_screen_init()
   char *fontname = NULL;
   char *widefontname = NULL;
   int len;
+  char *romfile = NULL;
 
   screen = DefaultScreen(display);
   color_map = DefaultColormap(display,screen);
@@ -501,17 +619,6 @@ void trs_screen_init()
       widefontname = malloc(len+1);
       strcpy(widefontname, def_widefont);
     }
-    /* Set default (kludge) */
-    trsfont = (strncasecmp(fontname, "-trs80", 6) == 0);
-  }
-
-  (void) sprintf(option, "%s%s", program_name, ".trsfont");
-  if (XrmGetResource(x_db, option, "Xtrs.Trsfont", &type, &value)) {
-    if (strcmp(value.addr,"on") == 0) {
-      trsfont = 1;
-    } else if (strcmp(value.addr,"off") == 0) {
-      trsfont = 0;
-    }
   }
 
   (void) sprintf(option, "%s%s", program_name, ".resize");
@@ -523,49 +630,64 @@ void trs_screen_init()
     }
   }
 
-  if (trs_model == 1) {
-    /* try resources first */
+  switch (trs_model) {
+  case 1:
     (void) sprintf(option, "%s%s", program_name, ".romfile");
     if (XrmGetResource(x_db, option, "Xtrs.Romfile", &type, &value)) {
-      trs_load_rom(value.addr);
+      if ((stat(value.addr, &statbuf)) == 0) { /* romfile exists */
+        romfile = value.addr;
+      }
+#ifdef DEFAULT_ROM
+    } else if ((stat(DEFAULT_ROM, &statbuf)) == 0) {
+      romfile = DEFAULT_ROM;
+#endif
+    }
+    if (romfile != NULL) {
+      trs_load_rom(romfile);
     } else if (trs_rom1_size > 0) {
       trs_load_compiled_rom(trs_rom1_size, trs_rom1);
     } else {
-#ifdef DEFAULT_ROM
-      trs_load_rom(DEFAULT_ROM);
-#else
-      fprintf(stderr,"%s: rom file not specified!\n",program_name);
-      exit(-1);
-#endif
+      fatal("ROM file not specified!");
     }
-  } else if (trs_model == 3 || trs_model == 4) {
+    break;
+  case 3: case 4:
     (void) sprintf(option, "%s%s", program_name, ".romfile3");
     if (XrmGetResource(x_db, option, "Xtrs.Romfile3", &type, &value)) {
-      trs_load_rom(value.addr);
+      if ((stat(value.addr, &statbuf)) == 0) { /* romfile exists */
+        romfile = value.addr;
+      }
+#ifdef DEFAULT_ROM3
+    } else if ((stat(DEFAULT_ROM3, &statbuf)) == 0) {
+      romfile = DEFAULT_ROM3;
+#endif
+    }
+    if (romfile != NULL) {
+      trs_load_rom(romfile);
     } else if (trs_rom3_size > 0) {
       trs_load_compiled_rom(trs_rom3_size, trs_rom3);
     } else {
-#ifdef DEFAULT_ROM3
-      trs_load_rom(DEFAULT_ROM3);
-#else
-      fprintf(stderr,"%s: rom file not specified!\n",program_name);
-      exit(-1);
-#endif
+      fatal("ROM file not specified!");
     }
-  } else {
+    break;
+  default: /* 4P */
     (void) sprintf(option, "%s%s", program_name, ".romfile4p");
     if (XrmGetResource(x_db, option, "Xtrs.Romfile4p", &type, &value)) {
-      trs_load_rom(value.addr);
+      if ((stat(value.addr, &statbuf)) == 0) { /* romfile exists */
+        romfile = value.addr;
+      }
+#ifdef DEFAULT_ROM4P
+    } else if ((stat(DEFAULT_ROM4P, &statbuf)) == 0) {
+      romfile = DEFAULT_ROM4P;
+#endif
+    }
+    if (romfile != NULL) {
+      trs_load_rom(romfile);
     } else if (trs_rom4p_size > 0) {
       trs_load_compiled_rom(trs_rom4p_size, trs_rom4p);
     } else {
-#ifdef DEFAULT_ROM4P
-      trs_load_rom(DEFAULT_ROM4P);
-#else
-      fprintf(stderr,"%s: rom file not specified!\n",program_name);
-      exit(-1);
-#endif
+      fatal("ROM file not specified!");
     }
+    break;
   }
 
   clear_key_queue();		/* init the key queue */
@@ -596,16 +718,14 @@ void trs_screen_init()
 
   if (usefont) {
     if ((myfont = XLoadQueryFont(display,fontname)) == NULL) {
-      fprintf(stderr,"%s: Can't open font %s!\n",program_name,fontname);
-      exit(-1);
+      fatal("can't open font %s!\n", fontname);
     }
     if ((mywidefont = XLoadQueryFont(display,widefontname)) == NULL) {
-      fprintf(stderr,"%s: Can't open font %s!\n",
-	      program_name,widefontname);
-      exit(-1);
+      fatal("can't open font %s!", widefontname);
     }
     curfont = myfont;
     XSetFont(display,gc,myfont->fid);
+    XSetFont(display,gc_inv,myfont->fid);
     cur_char_width =  myfont->max_bounds.width;
     cur_char_height = myfont->ascent + myfont->descent;
   }
@@ -625,7 +745,7 @@ void trs_screen_init()
   window = XCreateSimpleWindow(display, root_window, 400, 400,
 			       OrigWidth, OrigHeight, 1, foreground,
 			       background);
-  XStoreName(display,window,program_name);
+  XStoreName(display,window,title);
   XSelectInput(display, window, EVENT_MASK | ResizeRedirectMask);
   XMapWindow(display, window);
 
@@ -634,12 +754,12 @@ void trs_screen_init()
   screen_init();
   XClearWindow(display,window);
 
-#ifdef HAVE_SIGIO
+#if HAVE_SIGIO
   trs_event_init();
 #endif
 }
 
-#ifdef HAVE_SIGIO
+#if HAVE_SIGIO
 void trs_event_init()
 {
   int fd, rc;
@@ -692,7 +812,7 @@ void trs_get_event(int wait)
 
     switch(event.type) {
     case Expose:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"Expose\n");
 #endif
       if (event.xexpose.count == 0) {
@@ -702,7 +822,7 @@ void trs_get_event(int wait)
       break;
 
     case ResizeRequest:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"ResizeRequest w %d-->%d, h %d-->%d\n",
 	      OrigWidth, event.xresizerequest.width,
 	      OrigHeight, event.xresizerequest.height);
@@ -714,26 +834,26 @@ void trs_get_event(int wait)
 	XConfigureWindow(display,event.xresizerequest.window,
 			 (CWWidth | CWHeight),&xwc);
       }
-#ifdef DO_XFLUSH
+#if DO_XFLUSH
       XFlush(display);
 #endif
       break;
 
     case ConfigureNotify:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"ConfigureNotify\n");
 #endif
       break;
 
     case MapNotify:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"MapNotify\n");
 #endif
       trs_screen_refresh();
       break;
 
     case EnterNotify:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"EnterNotify\n");
 #endif
       save_repeat();
@@ -741,7 +861,7 @@ void trs_get_event(int wait)
       break;
 
     case LeaveNotify:
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"LeaveNotify\n");
 #endif
       restore_repeat();
@@ -750,9 +870,9 @@ void trs_get_event(int wait)
 
     case KeyPress:
       (void) XLookupString((XKeyEvent *)&event,buf,10,&key,&status);
-#ifdef XDEBUG
+#if XDEBUG
       fprintf(stderr,"KeyPress: state 0x%x, keycode 0x%x, key 0x%x\n",
-	      event.xkey.state, event.xkey.keycode, key);
+	      event.xkey.state, event.xkey.keycode, (unsigned int) key);
 #endif
       switch (key) {
 	/* Trap some function keys here */
@@ -799,14 +919,14 @@ void trs_get_event(int wait)
       if (key != 0) {
 	trs_xlate_keycode(0x10000 | key);
       }
-#ifdef XDEBUG
-      fprintf(stderr,"KeyRelease: keycode 0x%x, last_key 0x%x\n",
-	      event.xkey.state, event.xkey.keycode, key);
+#if XDEBUG
+      fprintf(stderr,"KeyRelease: state 0x%x, keycode 0x%x, last_key 0x%x\n",
+	      event.xkey.state, event.xkey.keycode, (unsigned int) key);
 #endif
       break;
 
     default:
-#ifdef XDEBUG	    
+#if XDEBUG	    
       fprintf(stderr,"Unhandled event: type %d\n", event.type);
 #endif
       break;
@@ -822,6 +942,7 @@ void trs_screen_expanded(int flag)
     if (usefont) {
       curfont = (flag ? mywidefont : myfont);
       XSetFont(display,gc,curfont->fid);
+      XSetFont(display,gc_inv,curfont->fid);
     }
     XClearWindow(display,window);
     trs_screen_refresh();
@@ -841,6 +962,19 @@ void trs_screen_inverse(int flag)
   }
 }
 
+void trs_screen_alternate(int flag)
+{
+  int bit = flag ? ALTERNATE : 0;
+  int i;
+  if ((currentmode ^ bit) & ALTERNATE) {
+    currentmode ^= ALTERNATE;
+    for (i = 0; i < screen_chars; i++) {
+      if (trs_screen[i] >= 0xc0)
+	trs_screen_write_char(i,trs_screen[i],False);
+    }
+  }
+}
+
 void trs_screen_640x240(int flag)
 {
   if (flag == screen640x240) return;
@@ -909,19 +1043,19 @@ boxes_init(int foreground, int background, int width, int height, int expanded)
 
   bits[0].y = bits[1].y = 0;
   bits[0].height = bits[1].height =
-    bits[2].y = bits[3].y = cur_char_height / 3;
-  bits[4].y = bits[5].y = (cur_char_height * 2) / 3;
+    bits[2].y = bits[3].y = height / 3;
+  bits[4].y = bits[5].y = (height * 2) / 3;
   bits[2].height = bits[3].height = bits[4].y - bits[2].y;
-  bits[4].height = bits[5].height = cur_char_height - bits[4].y;
+  bits[4].height = bits[5].height = height - bits[4].y;
 
-  for (graphics_char = 128; graphics_char < 192; ++graphics_char) {
-    trs_char[expanded][graphics_char] =
+  for (graphics_char = 0; graphics_char < 64; ++graphics_char) {
+    trs_box[expanded][graphics_char] =
       XCreatePixmap(display, window, width, height,
 		    DefaultDepth(display, screen));
 
     /* Clear everything */
     XSetForeground(display, gc, background);
-    XFillRectangle(display, trs_char[expanded][graphics_char],
+    XFillRectangle(display, trs_box[expanded][graphics_char],
 		   gc, 0, 0, width, height);
 
     /* Set the bits */
@@ -932,28 +1066,14 @@ boxes_init(int foreground, int background, int width, int height, int expanded)
 	cur_bits[p++] = bits[bit];
       }
     }
-    XFillRectangles(display, trs_char[expanded][graphics_char],
+    XFillRectangles(display, trs_box[expanded][graphics_char],
 		    gc, cur_bits, p);
   }
 }
 
 void bitmap_init(unsigned long foreground, unsigned long background)
 {
-  if(!usefont) {
-    /* Initialize from built-in font bitmaps. */
-    int i;
-	
-    for (i = 0; i < MAXCHARS; i++) {
-      trs_char[0][i] =
-	XCreateBitmapFromData(display,window,
-			      trs_char_data[trs_model != 1][i],
-			      TRS_CHAR_WIDTH,TRS_CHAR_HEIGHT);
-      trs_char[1][i] =
-	XCreateBitmapFromData(display,window,
-			      (char*)trs_widechar_data[trs_model!=1][i],
-			      TRS_CHAR_WIDTH*2,TRS_CHAR_HEIGHT);
-    }
-  } else if (!trsfont) {
+  if (usefont) {
     int dwidth, dheight;
 
     boxes_init(foreground, background,
@@ -967,6 +1087,24 @@ void bitmap_init(unsigned long foreground, unsigned long background)
       dheight = mywidefont->ascent + mywidefont->descent;
     }
     boxes_init(foreground, background, dwidth, dheight, 1);
+  } else {
+    /* Initialize from built-in font bitmaps. */
+    int i;
+	
+    for (i = 0; i < MAXCHARS; i++) {
+      trs_char[0][i] =
+	XCreateBitmapFromData(display,window,
+			      trs_char_data[trs_charset][i],
+			      TRS_CHAR_WIDTH,TRS_CHAR_HEIGHT);
+      trs_char[1][i] =
+	XCreateBitmapFromData(display,window,
+			      (char*)trs_widechar_data[trs_charset][i],
+			      TRS_CHAR_WIDTH*2,TRS_CHAR_HEIGHT);
+    }
+    boxes_init(foreground, background,
+	       cur_char_width, TRS_CHAR_HEIGHT, 0);
+    boxes_init(foreground, background,
+	       cur_char_width*2, TRS_CHAR_HEIGHT, 1);
   }
 }
 
@@ -1008,7 +1146,7 @@ void trs_screen_refresh()
       trs_screen_write_char(i,trs_screen[i],False);
     }
   }
-#ifdef DO_XFLUSH
+#if DO_XFLUSH
   XFlush(display);
 #endif
 }
@@ -1033,60 +1171,60 @@ void trs_screen_write_char(int position, int char_index, Bool doflush)
   col = position - (row * row_chars);
   destx = col * cur_char_width + left_margin;
   desty = row * cur_char_height + top_margin;
-  if (usefont) {
-    if (trs_model == 1 && !trsfont) {
-#ifndef UPPERCASE
+
+  if (trs_model == 1 && char_index >= 0xc0) {
+    /* On Model I, 0xc0-0xff is another copy of 0x80-0xbf */
+    char_index -= 0x40;
+  }
+  if (char_index >= 0x80 && char_index <= 0xbf && !(currentmode & INVERSE)) {
+    /* Use graphics character bitmap instead of font */
+    switch (currentmode & EXPANDED) {
+    case NORMAL:
+      XCopyArea(display,
+		trs_box[0][char_index-0x80],window,gc,0,0,
+		cur_char_width,cur_char_height,destx,desty);
+      break;
+    case EXPANDED:
+      /* use expanded graphics character bitmap instead of font */
+      XCopyArea(display,
+		trs_box[1][char_index-0x80],window,gc,0,0,
+		cur_char_width*2,cur_char_height,destx,desty);
+      break;
+    } 
+  } else if (usefont) {
+    /* Draw character using a font */
+    if (trs_model == 1) {
+#if !UPPERCASE
       /* Emulate Radio Shack lowercase mod.  The replacement character
 	 generator ROM had another copy of the uppercase characters in
 	 the control character positions, to compensate for a bug in the
 	 Level II ROM that stores such values instead of uppercase. */
       if (char_index < 0x20) char_index += 0x40;
 #endif
-      if (char_index >= 0x80) char_index &= 0xbf;
     }
-    if(!trsfont && char_index >= 0x80 && char_index <= 0xbf) {
-      /* use graphics character bitmap instead of font */
-      plane = 1;
-      switch (currentmode) {
-      case NORMAL:
-	XCopyArea(display,
-		  trs_char[0][char_index],window,gc,0,0,
-		  cur_char_width,cur_char_height,destx,desty);
-	break;
-      case EXPANDED:
-	XCopyArea(display,
-		  trs_char[1][char_index],window,gc,0,0,
-		  cur_char_width*2,cur_char_height,destx,desty);
-	break;
-      case INVERSE:
-	XCopyArea(display,
-		  trs_char[0][char_index & 0x7f], window,
-		  (char_index & 0x80) ? gc_inv : gc, 0, 0,
-		  cur_char_width,cur_char_height,destx,desty);
-	break;
-      case EXPANDED+INVERSE:
-	XCopyArea(display,
-		  trs_char[1][char_index & 0x7f], window,
-		  (char_index & 0x80) ? gc_inv : gc, 0, 0,
-		  cur_char_width*2,cur_char_height,destx,desty);
-	break;
-      }
+    if (trs_model > 1 && char_index >= 0xc0 &&
+	(currentmode & (ALTERNATE+INVERSE)) == 0) {
+      char_index -= 0x40;
+    }
+    desty += curfont->ascent;
+    if (currentmode & INVERSE) {
+      temp_char = (char)(char_index & 0x7f);
+      XDrawImageString(display, window,
+		       (char_index & 0x80) ? gc_inv : gc,
+		       destx, desty, &temp_char, 1);
     } else {
-      desty += curfont->ascent;
-      if (currentmode & INVERSE) {
-	temp_char = (char)(char_index & 0x7f);
-	XDrawImageString(display, window,
-			 (char_index & 0x80) ? gc_inv : gc,
-			 destx, desty, &temp_char, 1);
-      } else {
-	temp_char = (char)char_index;
-	XDrawImageString(display, window, gc,
-			 destx, desty, &temp_char, 1);
-      }
+      temp_char = (char)char_index;
+      XDrawImageString(display, window, gc,
+		       destx, desty, &temp_char, 1);
     }
   } else {
+    /* Draw character using a builtin bitmap */
+    if (trs_model > 1 && char_index >= 0xc0 &&
+	(currentmode & (ALTERNATE+INVERSE)) == 0) {
+      char_index -= 0x40;
+    }
     plane = 1;
-    switch (currentmode) {
+    switch (currentmode & ~ALTERNATE) {
     case NORMAL:
       XCopyPlane(display,trs_char[0][char_index],
 		 window,gc,0,0,TRS_CHAR_WIDTH,
@@ -1116,7 +1254,7 @@ void trs_screen_write_char(int position, int char_index, Bool doflush)
 	      (row*cur_char_height + grafyx_yoffset*2) % (2*G_YSIZE),
 	      destx, desty, TRS_CHAR_WIDTH, cur_char_height);
   }
-#ifdef DO_XFLUSH
+#if DO_XFLUSH
   if (doflush)
     XFlush(display);
 #endif
@@ -1148,7 +1286,7 @@ void trs_screen_write_chars(int *locations, int *values, int count)
   while (count--) {
     trs_screen_write_char(*locations++, *values++ , False);
   }
-#ifdef DO_XFLUSH
+#if DO_XFLUSH
   XFlush(display);
 #endif
 }
@@ -1280,6 +1418,11 @@ void grafyx_write_overlay(int value)
   }
 }
 
+int grafyx_get_microlabs()
+{
+  return grafyx_microlabs;
+}
+
 void grafyx_set_microlabs(int on_off)
 {
   grafyx_microlabs = on_off;
