@@ -24,14 +24,26 @@
 #include "config.h"
 #include <stdio.h>
 #include <ctype.h>
+#include <sys/time.h>
 
 #ifndef TRUE
 #define TRUE	(1)
 #define FALSE	(0)
 #endif
 
-typedef unsigned short Ushort;
-typedef unsigned char Uchar;
+typedef unsigned int Uint;      /* 4 bytes */
+typedef unsigned short Ushort;  /* 2 bytes */
+typedef unsigned char Uchar;    /* 1 byte */
+
+#ifdef NOTDEF /*__GNUC__*/
+typedef unsigned long long tstate_t;
+#define TSTATE_T_MID (((unsigned long long) -1LL)/2ULL)
+#define TSTATE_T_LEN "Lu"
+#else
+typedef unsigned long tstate_t;
+#define TSTATE_T_MID (((unsigned long) -1L)/2UL)
+#define TSTATE_T_LEN "lu"
+#endif
 
 struct twobyte
 {
@@ -42,6 +54,15 @@ struct twobyte
 #endif
 };
 
+struct fourbyte
+{
+#ifdef big_endian
+    Uchar byte3, byte2, byte1, byte0;
+#else
+    Uchar byte0, byte1, byte2, byte3;
+#endif
+};
+
 /* for implementing registers which can be seen as bytes or words: */
 typedef union
 {
@@ -97,20 +118,15 @@ struct z80_state_struct
     /* Speed control.  0 = full speed */
     int delay;
 
-    /* Simple event scheduler.  If >= 0, counts down before each
-     * instruction is executed.  When count goes from 0 to -1, 
-     * trs_do_event() is called.
-     */
-    int sched;
-
     /* Cyclic T-state counter */
-#ifdef __GNUC__
-    unsigned long long t_count;
-#else
-    unsigned long t_count;
-#endif
+    tstate_t t_count;
+
     /* Clock in MHz = T-states per microsecond */
     float clockMHz;
+
+    /* Simple event scheduler.  If nonzero, when t_count passes sched,
+     * trs_do_event() is called and sched is set to zero. */
+    tstate_t sched;
 };
 
 #define Z80_ADDRESS_LIMIT	(1 << 16)
@@ -153,8 +169,7 @@ struct z80_state_struct
 #define HIGH(p) (((struct twobyte *)(p))->high)
 #define LOW(p) (((struct twobyte *)(p))->low)
 
-#define T_COUNT(n) z80_state.t_count += (n)
-/*#define T_COUNT(n)*/
+#define T_COUNT(n) (z80_state.t_count += (n))
 
 /*
  * Flag accessors:
@@ -176,7 +191,9 @@ struct z80_state_struct
 #define SUBTRACT_MASK		(0x2)
 #define PARITY_MASK		(0x4)
 #define OVERFLOW_MASK		(0x4)
+#define UNDOC3_MASK             (0x8)
 #define HALF_CARRY_MASK		(0x10)
+#define UNDOC5_MASK             (0x20)
 #define ZERO_MASK		(0x40)
 #define	SIGN_MASK		(0x80)
 #define ALL_FLAGS_MASK		(CARRY_MASK | SUBTRACT_MASK | OVERFLOW_MASK | \
@@ -204,7 +221,6 @@ struct z80_state_struct
 #define PARITY_FLAG		(REG_F & PARITY_MASK)
 #define SUBTRACT_FLAG		(REG_F & SUBTRACT_MASK)
 #define CARRY_FLAG		(REG_F & CARRY_MASK)
-#define SIGN_FLAG		(REG_F & SIGN_MASK)
 
 extern struct z80_state_struct z80_state;
 
@@ -217,11 +233,11 @@ extern void mem_write_rom(int address, int value);
 extern int mem_read_word(int address);
 extern void mem_write_word(int address, int value);
 Uchar *mem_pointer(int address, int writing);
-extern void mem_block_transfer(Ushort dest, Ushort source, int direction,
-			       Ushort count);
+extern int mem_block_transfer(Ushort dest, Ushort source, int direction,
+			      Ushort count);
 extern int load_hex(); /* returns highest address loaded + 1 */
-extern void error(char *string);
-extern void fatal(char *string);
+extern void error(const char *fmt, ...);
+extern void fatal(const char *fmt, ...);
 extern void z80_out(int port, int value);
 extern int z80_in(int port);
 extern int disassemble(unsigned short pc);
